import React, { useState, useEffect, useRef } from 'react';
import { Outlet, history, useLocation, useAppData } from '@umijs/max';
import { NavBar, TabBar, List, ActionSheet, Popup } from 'antd-mobile';
import { UserOutline } from 'antd-mobile-icons';
import './index.less';
import { useMemo } from 'react';
import {
  MenuOutlined,
  ArrowLeftOutlined,
  EditOutlined,
  UsergroupAddOutlined,
  HomeOutlined,
  LogoutOutlined,
  LockOutlined,
  PieChartFilled,
} from '@ant-design/icons';
import { Drawer, Avatar, Button, Divider } from 'antd';
import { UserOutlined } from '@ant-design/icons';
import IconFont from '@/components/IconFont';
import { useRequest } from 'ahooks';
import { doFetch } from '../utils/doFetch';
import InitForm from '@/components/InitForm';
/* 
menulist 中的表示主路由菜单上左上角是打开 drawer 否则为返回上一页


*/

const menulist = ['/home'];

function BasicLayout() {
  const setRouteActive = (value) => {
    history.push(value);
  };
  const { data, loading, refreshAsync } = useRequest(async () => {
    let res = await doFetch({ url: '/app/user/me', params: {} });
    return res?.data?.data;
  }, {});

  const tabs = [
    {
      key: '/home',
      title: '设备',
      icon: <IconFont type="icon-pintu" />,
    },
    {
      key: '/todo',
      title: '告警',
      icon: <IconFont type="icon-bell" />,
    },
    {
      key: '/message',
      title: '统计',
      icon: <PieChartFilled />,
    },
  ];
  const location = useLocation();
  const { pathname } = location;

  const { routes } = useAppData();

  const currouter = useMemo(() => {
    let allroutes = Object.values(routes).filter((it, i) => {
      return it.path === pathname;
    })?.[0];
    return allroutes;
  }, [pathname]);

  const [drawer, setdrawer] = useState({ open: false });

  const [startX, setStartX] = useState(null);
  const [startY, setStartY] = useState(null);

  function handleTouchStart(event) {
    setStartX(event.touches[0].clientX);
    setStartY(event.touches[0].clientY);
  }

  function handleTouchMove(event) {
    const { clientX, clientY } = event.touches[0];
    const distance = clientX - startX;
    const distancey = clientY - startY;
    if (distancey > 10 || distancey < -10) {
      return;
    }
    if (distance > 20) {
      // 执行左滑操作
      setdrawer((s) => ({
        ...s,
        open: true,
      }));
    }
  }
  const handler = useRef(),formRef = useRef();
  const [visible1, setVisible1] = useState(false);
  const [type, settype] = useState({});

  return (
    <div className="container">
      <Drawer
        onClose={() => {
          setdrawer((s) => ({
            ...s,
            open: false,
          }));
        }}
        closable={false}
        {...drawer}
        width={'80%'}
        placement="left"
      >
        <div
          className="userinfo"
          style={{ position: 'relative', height: '100%' }}
        >
          <Popup
            visible={visible1}
            onMaskClick={() => {
              setVisible1(false);
            }}
            bodyStyle={{ height: type?.val === 'info' ?'330px':'430px' }}
            position="top"
          >
            {type?.val === 'info' ? (
              <div className="popcontian">
                <h2>修改姓名/手机号</h2>
                <InitForm
                  fields={[
                    {
                      title: '姓名',
                      dataIndex: 'fullName',
                      key: 'fullName',
                    },
                    {
                      title: '手机号',
                      dataIndex: 'phone',
                      key: 'phone',
                    },
                  ]}
                  defaultFormValue={{
                    ...data,
                  }}
                  onFinish={async (vals) => {
                    let res = await doFetch({
                      url: '/app/user/updateInfo',
                      params: { ...vals, id: data?.id },
                    });
                    if (res?.code === '0000') {
                      await refreshAsync();
                      setVisible1(false);
                    }
                  }}
                ></InitForm>
              </div>
            ) : (
              <div className="popcontian">
                <h2>修改密码</h2>
                <InitForm
                  formRef={formRef}
                  fields={[
                    {
                      "title": "原始密码",
                      "dataIndex": "oldPassword",
                      "key": "oldPassword",
                      "formItemProps": {
                        "rules": [
                          {
                            "required": true,
                            "message": "此项为必填项"
                          }
                        ]
                      }
                    },
                    {
                      "title": "新密码",
                      "dataIndex": "password",
                      "key": "password",
                      "formItemProps": {
                        "rules": [
                          {
                            "required": true,
                            "message": "此项为必填项"
                          }
                        ]
                      },
                      "valueType": "password"
                    },
                    {
                      "title": "确认新密码",
                      "dataIndex": "confirmPassword",
                      "key": "confirmPassword",
                      "formItemProps": {
                        "rules": [
                          {
                            "required": true,
                            "message": "此项为必填项"
                          },
                          ({ getFieldValue }) => ({
                            validator(_, value) {
                              if (!value || getFieldValue('password') === value) {
                                return Promise.resolve();
                              }
                              return Promise.reject(new Error('两次输入的密码不一致!'));
                            },
                          }),
                        ]
                      },
                      "valueType": "password"
                    }
                  ]}
                  onFinish={async (vals) => {
                    let res = await doFetch({
                      url: '/app/user/updatePassword',
                      params: { ...vals, id: data?.id },
                    });
                    if (res?.code === '0000') {
                      await refreshAsync();
                      setVisible1(false);
                    }
                  }}
                ></InitForm>
              </div>
            )}
          </Popup>
          


          <Avatar size={56} icon={<UserOutlined />}></Avatar>
          <div
            className="spread"
            style={{ alignItems: 'flex-end', paddingBottom: 8 }}
          >
            <div>
              <h2>{data?.fullName}</h2>
              <p style={{ margin: 0 }}>{data?.phone || '未填写手机号'}</p>
            </div>
            <Button
              style={{ color: '#fff', borderColor: '#fff' }}
              ghost
              type="primary"
              icon={<EditOutlined />}
              onClick={() => {
                settype({
                  val: 'info',
                });
                setVisible1(true);
              }}
            ></Button>
          </div>
          <Divider></Divider>
          <List header="个人信息" style={{ marginLeft: -12 }}>
            <List.Item prefix={<UserOutline />} extra={data?.username}>
              用户名
            </List.Item>
            <List.Item
              prefix={<UsergroupAddOutlined />}
              extra={data?.orgNames || '-'}
            >
              组织
            </List.Item>
            <List.Item
              prefix={<HomeOutlined style={{ fontSize: 14, paddingLeft: 3 }} />}
              extra={data?.companyName || '-'}
            >
              公司
            </List.Item>
            <div style={{alignItems:"flex-end",paddingLeft:8}} className="center">
              <Divider style={{minWidth:120,width:"calc(100% - 0px)",marginBottom:16}}></Divider>
            </div>
            <List.Item
              prefix={<LockOutlined />}
              onClick={() => {
                settype({
                  val: 'pwd',
                });
                setVisible1(true);
              }}
            >
              修改密码
            </List.Item>
          </List>
          <Button
            icon={<LogoutOutlined />}
            size="large"
            style={{
              width: '100%',
              marginTop: 12,
              position: 'absolute',
              bottom: 0,
              left: 0,
              right: 0,
              backgroundColor: 'rgba(0,0,0,0.2)',
            }}
            type="ghost"
            onClick={() => {
              handler.current = ActionSheet.show({
                actions: [
                  {
                    text: '确认退出',
                    key: 'copy',
                    danger: true,
                    bold: true,
                    onClick: () => {
                      history.replace('/login');
                      handler.current.close();
                    },
                  },
                  {
                    text: '取消退出',
                    key: 'edit',
                    onClick: () => {
                      handler.current.close();
                    },
                  },
                ],
              });
            }}
          >
            退出
          </Button>
        </div>
      </Drawer>
      <div className="top">
        <NavBar
          backArrow={
            menulist.includes(pathname) ? (
              <IconFont type="icon-menu" style={{ color: '#fff' }} />
            ) : (
              <ArrowLeftOutlined />
            )
          }
          onBack={() => {
            if (menulist.includes(pathname)) {
              setdrawer((s) => ({
                ...s,
                open: true,
              }));
            } else {
              history.go(-1);
            }
          }}
        >
          {currouter?.name}
        </NavBar>
      </div>
      <div style={{ height: 94 }}></div>
      <div onTouchStart={handleTouchStart} onTouchMove={handleTouchMove}>
        <Outlet></Outlet>
      </div>

      <div style={{ height: 66 }}></div>
      <div className="bottom">
        <TabBar
          activeKey={pathname}
          onChange={(value) => setRouteActive(value)}
        >
          {tabs.map((item) => (
            <TabBar.Item key={item.key} icon={item.icon} title={item.title} />
          ))}
        </TabBar>
      </div>
    </div>
  );
}

export default BasicLayout;
