/* eslint-disable eqeqeq */
import * as React from 'react';
import { useState, useMemo, useRef } from 'react';
import DrawerPro from '@/components/DrawerPro';
import AutoTable from '@/components/AutoTable';
import { Badge } from 'antd';
import getcolumns from './columns';
import { useAsyncEffect, useRequest } from 'ahooks';
import { doFetch } from '@/utils/doFetch';
import ReactECharts from 'echarts-for-react';
import Settings from '../../../config/defaultSettings';

const { proxypath } = Settings;

function cropImage(url) {
  return new Promise((resolve) => {
    const img = new Image();
    img.crossOrigin = 'Anonymous';
    img.src = url;
    img.onload = () => {
      const canvas = document.createElement('canvas');
      const ctx = canvas.getContext('2d');
      const size = Math.min(img.width, img.height);
      canvas.width = size;
      canvas.height = size;
      ctx.beginPath();
      ctx.arc(size / 2, size / 2, size / 2, 0, 2 * Math.PI);
      ctx.clip();
      ctx.drawImage(
        img,
        (img.width - size) / 2,
        (img.height - size) / 2,
        size,
        size,
        0,
        0,
        size,
        size,
      );
      canvas.toBlob((blob) => {
        const reader = new FileReader();
        reader.readAsDataURL(blob);
        reader.onloadend = () => {
          resolve(reader.result);
        };
      }, 'image/png');
    };
  });
}

const Cards = ({ item, columns }) => {
  const [option, setoption] = useState({});
  const { statusColorList, deviceStatusDurationList } = item;
  useAsyncEffect(async () => {
    let img =
      item?.image?.length > 0
        ? proxypath + item.image?.[0]?.url
        : require('@/assets/default.png');
    const image = await cropImage(img);
    let normalColor =
      statusColorList.filter((el) => el.type == 3)?.[0]?.color ?? '';
    let data = deviceStatusDurationList?.map((it) => {
      let label =
          statusColorList.filter((el) => el.type == it.name)?.[0]?.name ?? '',
        itemColor =
          statusColorList.filter((el) => el.type == it.name)?.[0]?.color ?? '';
      return {
        ...it,
        label,
        itemStyle: {
          color: itemColor,
        },
      };
    }) ?? [
      {
        name: '关机',
        value: 1,
        itemStyle: {
          color: normalColor,
        },
      },
    ];

    const options = {
      series: [
        {
          type: 'pie',
          radius: ['85%', '95%'],
          avoidLabelOverlap: false,
          label: {
            show: false,
            position: 'center',
          },
          data,
        },
      ],
      graphic: {
        elements: [
          {
            type: 'image',
            style: {
              image: image,
              width: 88,
              height: 88,
            },
            left: 'center',
            top: 'center',
          },
        ],
      },
    };

    setoption(options);
  }, [item]);

  return (
    <div className="split">
      <div className="spread" style={{ justifyContent: 'flex-start' }}>
        <Badge
          count={item.alarmNum}
          overflowCount={99}
          offset={[-10, 10]}
          color={
            item.alarmNum < 10
              ? '#108ee9'
              : item.alarmNum < 30
              ? '#f76e1e'
              : 'red'
          }
        >
          <div style={{ width: 120, height: 120, flexShrink: 0 }}>
            <ReactECharts option={option} style={{ height: '100%' }} />
          </div>
        </Badge>

        <div style={{ flex: 1 }} className="rightsection">
          {columns
            ?.filter((it) => !(it.valueType === 'split' || it?.hideInTable))
            ?.map((it, i) => {
              const Bod = ({ children }) =>
                i == 0 ? (
                  <h2>{children}</h2>
                ) : i == 1 ? (
                  <h3>{children}</h3>
                ) : (
                  <p>{children}</p>
                );
              return (
                <Bod key={it?.dataIndex}>
                  {it?.hidetitle ? '' : `${it?.title}: `}
                  {it.render
                    ? it.render(item[it?.dataIndex], item)
                    : item[it?.dataIndex]}
                </Bod>
              );
            })}
        </div>
      </div>
    </div>
  );
};

function Home(props) {
  const actionRef = useRef(),
    formRef = useRef();
  const [drawer, setdrawer] = useState({
      open: false,
    }),
    [activeTabKey, setactiveTabKey] = useState('1');

  const { run, loading } = useRequest(doFetch, {
    manual: true,
    onSuccess: (res, params) => {
      if (res?.code === '0000') {
        actionRef?.current?.reload();
        setdrawer((s) => ({
          ...s,
          open: false,
        }));
      }
    },
  });

  const columns = useMemo(() => {
    let defcolumn = getcolumns(setdrawer)?.columns;
    return defcolumn;
  }, [activeTabKey]);

  const pathconfig = useMemo(() => {
    let defpath = getcolumns()?.pathconfig ?? {};
    return defpath;
  }, [activeTabKey]);

  return (
    <div style={{ position: 'relative' }}>
      <AutoTable
        columns={columns}
        path={'/app/device/index'}
        actionRef={actionRef}
      >
        <Cards />
      </AutoTable>

      <DrawerPro
        fields={columns}
        detailpath={pathconfig?.detail || null}
        detailData={drawer?.item}
        defaultFormValue={drawer?.item}
        params={{ id: drawer?.item?.id }}
        formRef={formRef}
        placement="right"
        onClose={() => {
          setdrawer((s) => ({
            ...s,
            open: false,
          }));
        }}
        {...drawer}
        onFinish={(vals) => {
          if (drawer?.val === 'add') {
            run({ url: pathconfig?.add || '/add', params: { ...vals } });
          } else if (drawer?.val === 'edit') {
            run({
              url: pathconfig?.edit || '/edit',
              params: { ...vals, id: drawer?.item?.id },
            });
          }
        }}
      />
    </div>
  );
}

export default Home;
